package edu.umich.ac;

import android.content.Context;
import android.graphics.Bitmap;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class Preview extends SurfaceView implements SurfaceHolder.Callback {
    SurfaceHolder mHolder;
    Camera mCamera;
    Bitmap mBitmap;
    EffectView mEffectView;
    
    public static final int RGB_565 = 0x00000004;
    public static final int NV21 = 0x00000011;
    public static final int RGBA_8888 = 0x00000001;
    
    public Preview(Context context, EffectView effectView) {
        super(context);
        
        // Install a SurfaceHolder.Callback so we get notified when the
        // underlying surface is created and destroyed.
        mHolder = getHolder();
        mHolder.addCallback(this);
        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
        
        mEffectView = effectView;
    }
    
    public Preview(Context context) {
    	super(context);
    	mHolder = getHolder();
    	mHolder.addCallback(this);
    	mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }
    
    public Preview(Context context, AttributeSet attrs) {
    	super(context, attrs);
    	mHolder = getHolder();
    	mHolder.addCallback(this);
    	mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }
    
    
    public void setEffectView(EffectView ev) {
    	mEffectView = ev;
    }

    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created, acquire the camera and tell it where
        // to draw.
        mCamera = Camera.open();
        try {
           mCamera.setPreviewDisplay(holder);
           mCamera.setPreviewCallback(new Camera.PreviewCallback() {
			
			public void onPreviewFrame(byte[] data, Camera camera) {
				/*mBitmap = Bitmap.createBitmap((int[])data, camera.getParameters().getPictureSize().width, 
						camera.getParameters().getPreviewSize().height, Bitmap.Config.ARGB_8888);
				Preview.this.invalidate();*/
				if (mEffectView != null) {
					int width = camera.getParameters().getPreviewSize().width;
					int height = camera.getParameters().getPreviewSize().height;
					int [] rgb = new int[width*height];
					decodeYUV420SP(rgb, data, width, height);
					/*mBitmap = BitmapFactory.decodeByteArray(data, 0, data.length);
					mBitmap = mBitmap.copy(Bitmap.Config.ARGB_8888, true);*/
					//mBitmap = Bitmap.createBitmap(rgb, width, height, Bitmap.Config.RGB_565);
					//mEffectView.setImage(mBitmap);
					mEffectView.setSize(width, height);
					mEffectView.setColors(rgb);
					mEffectView.invalidate();
				}
			}
		});
        } catch (Exception exception) {
            mCamera.release();
            mCamera = null;
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return, so stop the preview.
        // Because the CameraDevice object is not a shared resource, it's very
        // important to release it when the activity is paused.
        mCamera.stopPreview();
        mCamera.release();
        mCamera = null;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Now that the size is known, set up the camera parameters and begin
        // the preview.
        Camera.Parameters parameters = mCamera.getParameters();
        //parameters.setPreviewSize(w, h);
        parameters.setPreviewSize(160, 120);
        parameters.setPreviewFrameRate(15);
        //parameters.setPreviewFormat(20);
        //int i = parameters.getPreviewFormat();
        mCamera.setParameters(parameters);
        mCamera.startPreview();
    }
    
    static public void decodeYUV420SP(int[] rgb, byte[] yuv420sp, int width, int height) {
    	final int frameSize = width * height;
    	
    	for (int j = 0, yp = 0; j < height; j++) {
    		int uvp = frameSize + (j >> 1) * width, u = 0, v = 0;
    		for (int i = 0; i < width; i++, yp++) {
    			int y = (0xff & ((int) yuv420sp[yp])) - 16;
    			if (y < 0) y = 0;
    			if ((i & 1) == 0) {
    				v = (0xff & yuv420sp[uvp++]) - 128;
    				u = (0xff & yuv420sp[uvp++]) - 128;
    			}
    			
    			int y1192 = 1192 * y;
    			int r = (y1192 + 1634 * v);
    			int g = (y1192 - 833 * v - 400 * u);
    			int b = (y1192 + 2066 * u);
    			
    			if (r < 0) r = 0; else if (r > 262143) r = 262143;
    			if (g < 0) g = 0; else if (g > 262143) g = 262143;
    			if (b < 0) b = 0; else if (b > 262143) b = 262143;
    			
    			rgb[yp] = 0xff000000 | ((r << 6) & 0xff0000) | ((g >> 2) & 0xff00) | ((b >> 10) & 0xff);
    		}
    	}
    }

}
